<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Block\Adminhtml\Setup\Index;

use WriteTextAI\WriteTextAI\Block\Adminhtml\Setup\Index;
use Magento\Backend\Block\Template\Context;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Model\GenerateOptionsManager;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Helper\Categories\Fields as CategoryFields;
use WriteTextAI\WriteTextAI\Model\Config\Source\CategoryTexts;
use WriteTextAI\WriteTextAI\Model\Config\Source\ProductTexts;
use WriteTextAI\WriteTextAI\Model\Config\Source\NotifacationEmails;
use WriteTextAI\WriteTextAI\Model\Config\Source\TriggerActions;
use WriteTextAI\WriteTextAI\Model\Config\Source\AutomaticTextOptimization;
use WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\CategoryTexts as DefaultCategoryTexts;
use WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\ProductTexts as DefaultProductTexts;
use WriteTextAI\WriteTextAI\ViewModel\Premium;

class Keywords extends Index
{
    /**
     * @var CategoryTexts
     */
    protected $categoryTexts;

    /**
     * @var ProductTexts
     */
    protected $productTexts;

    /**
     * @var NotifacationEmails
     */
    protected $notifacationEmails;

    /**
     * @var TriggerActions
     */
    protected $triggerActions;

    /**
     * @var DefaultCategoryTexts
     */
    protected $defaultCategoryTexts;

    /**
     * @var DefaultProductTexts
     */
    protected $defaultProductTexts;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * Constructor
     *
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \WriteTextAI\WriteTextAI\Helper\Data $helper
     * @param \WriteTextAI\WriteTextAI\Model\GenerateOptionsManager $generateOptionsManager
     * @param \WriteTextAI\WriteTextAI\Model\ApiManager $apiManager
     * @param \WriteTextAI\WriteTextAI\Model\SettingManager $settingManager
     * @param \WriteTextAI\WriteTextAI\Helper\Categories\Fields $categoryFields
     * @param \WriteTextAI\WriteTextAI\Model\Config\Source\CategoryTexts $categoryTexts
     * @param \WriteTextAI\WriteTextAI\Model\Config\Source\ProductTexts $productTexts
     * @param \WriteTextAI\WriteTextAI\Model\Config\Source\NotifacationEmails $notifacationEmails
     * @param \WriteTextAI\WriteTextAI\Model\Config\Source\TriggerActions $triggerActions
     * @param \WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\CategoryTexts $defaultCategoryTexts
     * @param \WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\ProductTexts $defaultProductTexts
     * @param \WriteTextAI\WriteTextAI\ViewModel\Premium $premium
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        GenerateOptionsManager $generateOptionsManager,
        ApiManager $apiManager,
        SettingManager $settingManager,
        CategoryFields $categoryFields,
        CategoryTexts $categoryTexts,
        ProductTexts $productTexts,
        NotifacationEmails $notifacationEmails,
        TriggerActions $triggerActions,
        DefaultCategoryTexts $defaultCategoryTexts,
        DefaultProductTexts $defaultProductTexts,
        Premium $premium,
        array $data = []
    ) {
        $this->categoryTexts = $categoryTexts;
        $this->productTexts = $productTexts;
        $this->notifacationEmails = $notifacationEmails;
        $this->triggerActions = $triggerActions;
        $this->defaultCategoryTexts = $defaultCategoryTexts;
        $this->defaultProductTexts = $defaultProductTexts;
        $this->premium = $premium;
        parent::__construct(
            $context,
            $helper,
            $generateOptionsManager,
            $apiManager,
            $settingManager,
            $categoryFields,
            $data
        );
    }

    /**
     * Get premium
     *
     * @return \WriteTextAI\WriteTextAI\ViewModel\Premium
     */
    public function getPremium()
    {
        return $this->premium->isPremium();
    }

    /**
     * Get category fields
     *
     * @return array
     */
    public function getCategoryFields()
    {
        return $this->categoryTexts->toOptionArray();
    }

    /**
     * Get product fields
     *
     * @return array
     */
    public function getProductFields()
    {
        return $this->productTexts->toOptionArray();
    }

    /**
     * Get notification emails
     *
     * @return array
     */
    public function getNotificationEmails()
    {
        return $this->notifacationEmails->toOptionArray();
    }

    /**
     * Get automatic text optimization
     *
     * @return array
     */
    public function getAutomaticTextOptimization()
    {
        return $this->helper->getAutomaticTextOptimizationOptions();
    }

    /**
     * Get trigger actions
     *
     * @return array
     */
    public function getTriggerActions()
    {
        return $this->triggerActions->toOptionArray();
    }

    /**
     * Get default cluster count
     *
     * @return int
     */
    public function getDefaultClusterCount()
    {
        $defaultCount = $this->helper->getKeywordOptimizationSettings('cluster_based_pipelines_count');
        if ($defaultCount !== null) {
            return (int) $defaultCount;
        }
        $rules = $this->getRules();
        return $rules['minClusterBasedPipelinesCount'] ?? 2;
    }

    /**
     * Get default category texts
     *
     * @return array
     */
    public function getDefaultCategoryTexts()
    {
        $selected = $this->helper->getKeywordOptimizationSettings('categories');
        if ($selected) {
            return explode(',', $selected);
        }

        $defaultCategoryTexts = $this->defaultCategoryTexts->getDefaultValues();

        return explode(',', $defaultCategoryTexts);
    }

    /**
     * Get default product texts
     *
     * @return array
     */
    public function getDefaultProductTexts()
    {
        $selected = $this->helper->getKeywordOptimizationSettings('products');
        if ($selected) {
            return explode(',', $selected);
        }

        $defaultProductTexts = $this->defaultProductTexts->getDefaultValues();

        return explode(',', $defaultProductTexts);
    }

    /**
     * Get default trigger action
     *
     * @return string
     */
    public function getDefaultTriggerAction()
    {
        $selected = $this->helper->getKeywordOptimizationSettings('pipeline_keyword/low/trigger_action');
        if ($selected) {
            return $selected;
        }
        return TriggerActions::WHEN_PAGE_IS_RANKING;
    }

    /**
     * Get default automatic text optimization
     *
     * @return string
     */
    public function getDefaultAutomaticTextOptimization()
    {
        $selected = $this->helper->getKeywordOptimizationSettings('automatic_text_optimization');
        if ($selected) {
            return $selected;
        }
        return AutomaticTextOptimization::AUTO_REWRITE_AND_QUEUE;
    }

    /**
     * Get trigger action n position default value
     *
     * @return string
     */
    public function getTriggerActionNPositionDefaultValue()
    {
        $defaultVal = $this->helper->getKeywordOptimizationSettings('pipeline_keyword/low/trigger_action_n_position');
        if ($defaultVal !== null) {
            return $defaultVal;
        }
        $rules = $this->getRules();
        return $rules['minTriggerActionNPosition'] ?? 1;
    }

    /**
     * Get trigger action n position default value per difficulty
     *
     * @param string $difficulty
     *
     * @return string
     */
    public function getTriggerActionNPositionDefaultValuePerDifficulty($difficulty)
    {
        $settingDifficulty = $difficultySettingsEquivalent[$difficulty] ?? 'low';
        $defaultVal = $this->helper->getKeywordOptimizationSettings('pipeline_keyword');
        if (isset($defaultVal[$difficulty]['trigger_action_n_position'])) {
            return $defaultVal[$difficulty]['trigger_action_n_position'];
        }
        $rules = $this->getRules();
        return $rules['minTriggerActionNPositionPerDifficulty'] ?? 1;
    }

    /**
     * Get trigger action after n days default value
     *
     * @return string
     */
    public function getTriggerActionNDaysDefaultValue()
    {
        $defaultVal = $this->helper->getKeywordOptimizationSettings('pipeline_keyword/low/trigger_action_n_days');
        if ($defaultVal !== null) {
            return $defaultVal;
        }
        $rules = $this->getRules();
        return $rules['minTriggerActionNPosition'] ?? 1;
    }

    /**
     * Get trigger action after n days default value per difficulty
     *
     * @param string $difficulty
     *
     * @return string
     */
    public function getTriggerActionNDaysDefaultValuePerDifficulty($difficulty)
    {
        $settingDifficulty = $difficultySettingsEquivalent[$difficulty] ?? 'low';
        $defaultVal = $this->helper->getKeywordOptimizationSettings('pipeline_keyword');
        if (isset($defaultVal[$difficulty]['trigger_action_n_days'])) {
            return $defaultVal[$difficulty]['trigger_action_n_days'];
        }
        $rules = $this->getRules();
        return $rules['minTriggerActionNDaysPerDifficulty'] ?? 30;
    }

    /**
     * Get trigger action page is ranking value string
     *
     * @return string
     */
    public function getTriggerActionPageIsRankingValue()
    {
        return TriggerActions::WHEN_PAGE_IS_RANKING;
    }

    /**
     * Get trigger action after N days value string
     *
     * @return string
     */
    public function getTriggerActionAfterNDaysValue()
    {
        return TriggerActions::AFTER_N_DAYS;
    }
}
