<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Magento\Framework\App\ResourceConnection;

class ResetData extends Command
{
    /**
     * @var ResourceConnection
     */
    private $resource;

    /**
     * Constructor.
     *
     * @param ResourceConnection $resource
     * @param string|null $name
     */
    public function __construct(
        ResourceConnection $resource,
        string $name = null
    ) {
        parent::__construct($name);
        $this->resource = $resource;
    }

    /**
     * @inheritdoc
     */
    protected function configure()
    {
        $this->setName('writetextai:data_reset')
             ->setDescription('Reset database data');
    }

    /**
     * @inheritdoc
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        try {
            $output->writeln('Start Reset');

            $timeStart = microtime(true);
            
            $connection = $this->resource->getConnection();
            $this->dropTables($connection);
            $this->deleteConfigData($connection);

            $timeLasted = ceil(microtime(true) - $timeStart);

            $output->writeln('End Reset, Time Lasted: ' . $timeLasted . ' secs');
        } catch (\Exception $e) {
            $output->writeln('Error: ' . $e->getMessage());
        }

        return 0;
    }

    /**
     * Drop tables
     *
     * @param \Magento\Framework\DB\Adapter\AdapterInterface $connection
     */
    protected function dropTables($connection)
    {
        $tablesToDrop = [
            "writetextai_writetextai_products_store",
            "writetextai_writetextai_categories_store",
            "writetextai_writetextai_settings",
            "writetextai_writetextai_user_fields_selected",
            "writetextai_writetextai_user_web_token",
            "writetextai_writetextai_bulk_generate_requests",
            "writetextai_writetextai_bulk_transfer_requests",
            "writetextai_writetextai_bulk_keyword_analysis_requests",
            "writetextai_writetextai_user_feedback",
            "writetextai_writetextai_single_generate_requests",
            "writetextai_writetextai_product_reference_product",
            "writetextai_writetextai_product_keywords",
            "writetextai_writetextai_user_request_status",
            "writetextai_writetextai_product_keyword_ideas_filters",
            "writetextai_writetextai_product_keyword_ranked_filters",
            "writetextai_writetextai_product_keyword_competitor_filters"
        ];

        foreach ($tablesToDrop as $table) {
            $tableName = $this->resource->getTableName($table);
            if ($connection->isTableExists($tableName)) {
                $connection->truncateTable($tableName);
            }
        }
    }

    /**
     * Delete config data
     *
     * @param \Magento\Framework\DB\Adapter\AdapterInterface $connection
     */
    protected function deleteConfigData($connection)
    {
        $configTable = $this->resource->getTableName('core_config_data');
        $connection->delete($configTable, "`path` LIKE 'writetextai_settings%'");
    }
}
