<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Ui\Component\MassAction\Filter;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Store\Model\Store;

class GetImages extends Action implements HttpPostActionInterface
{
    /**
     * @var Filter
     */
    protected $filter;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * Constructor
     *
     * @param Context $context
     * @param Filter $filter
     * @param CollectionFactory $collectionFactory
     * @param Premium $premium
     */
    public function __construct(
        Context $context,
        Filter $filter,
        CollectionFactory $collectionFactory,
        Premium $premium
    ) {
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        $this->premium = $premium;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $result = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $result->setData([
                'success' => false,
                'reload' => true,
                'message' => __('Premium is required to do this action.')
            ]);

            return $result;
        }
        
        try {
            $filters = $this->getRequest()->getPost('filters', []);

            $storeFilter = isset($filters['store_id']) ? $filters['store_id'] : Store::DEFAULT_STORE_ID;

            $collection = $this->filter
                ->getCollection(
                    $this->collectionFactory
                        ->create()
                        ->addAttributeToSelect('*')
                )
                ->setStoreId($storeFilter)
                ->addMediaGalleryData();
            
            $images = [];
            foreach ($collection as $product) {
                $mediaGallery = $product->getMediaGalleryImages()->setOrder('position', 'ASC');
                if (!empty($mediaGallery)) {
                    $mediaGalleryIds = [];
                    foreach ($mediaGallery as $mediaGalleryEntry) {
                        if (!$mediaGalleryEntry->getDisabled()) {
                            $mediaGalleryIds[] = $mediaGalleryEntry->getId();
                        }
                    }
    
                    if (!empty($mediaGalleryIds)) {
                        $images[$product->getId()] = implode(",", $mediaGalleryIds);
                    }
                }
            }

            $resultData = [
                'success' => true,
                'images' => $images
            ];

            $result->setData($resultData);
        } catch (\Exception $e) {
            $message = $e->getMessage();
            if (!$message) {
                $message = __(
                    'A system error has occurred. Please try again. If the issue persists,'
                    . ' please contact our support team at support@writetext.ai.'
                );
            }
            
            $result->setData([
                'success' => false,
                'message' => $message
            ]);
        }

        return $result;
    }
}
