<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Settings;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use Magento\Framework\App\Config\ConfigResource\ConfigInterface;
use WriteTextAI\WriteTextAI\Helper\Data;

class Popups extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var ConfigInterface
     */
    protected $configManager;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param ConfigInterface $configManager
     * @param Data $helper
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        ConfigInterface $configManager,
        Data $helper
    ) {
        $this->apiManager = $apiManager;
        $this->configManager = $configManager;
        $this->helper = $helper;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $params = $this->getRequest()->getParams();

            switch ($params['type']) {
                case 'new_to_premium':
                    $confPath = 'writetextai_settings/popups/new_to_premium';
                    $this->apiManager->setWelcomePremium20ToFalse();
                    break;
                case 'update_to_v2':
                    $confPath = 'writetextai_settings/popups/update_to_v2';
                    $this->apiManager->setWelcomePremium20ToFalse();
                    break;
                case 'update_to_v21':
                    $confPath = 'writetextai_settings/popups/update_to_v21';
                    break;
            }

            $this->configManager->saveConfig($confPath, 1, 'default');
            $this->helper->flushConfigCache();
            
            $result = [
                'success' => true
            ];
        } catch (\Exception $e) {
            $message = $e->getMessage();
            if (!$message) {
                $message = __(
                    'A system error has occurred. Please try again. If the issue persists,'
                    . ' please contact our support team at support@writetext.ai.'
                );
            }

            $result = [
                'success' => false,
                'message' => $message
            ];
        }

        $response->setData($result);

        return $response;
    }
}
