<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Settings;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;

class SetFreePremiumToFalse extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager
    ) {
        $this->apiManager = $apiManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $setFreePremium = $this->apiManager->setFreePremiumCreditsToFalse();
            $credit = $this->apiManager->getCredit();
            
            $result = [
                'success' => true,
                'is_free_premium_credits' => isset($credit['freePremiumCredits']) ?
                    $credit['freePremiumCredits'] : false,
                'is_already_premium' => isset($credit['freeCreditsAlreadyPremium']) ?
                    $credit['freeCreditsAlreadyPremium'] : false,
                'message' => 'Free Premium Credits set to false'
            ];
        } catch (\Exception $e) {
            $message = $e->getMessage();
            if (!$message) {
                $message = __(
                    'A system error has occurred. Please try again. If the issue persists,'
                    . ' please contact our support team at support@writetext.ai.'
                );
            }

            $result = [
                'success' => false,
                'is_free_premium_credits' => false,
                'message' => $message
            ];
        }

        $response->setData($result);

        return $response;
    }
}
