<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Plugin;

use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\App\Action\Action;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\Response\Http;

class TotalCreditsPlugin
{
    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var ResultFactory
     */
    protected $resultFactory;

    /**
     * @var Http
     */
    protected $httpResponse;

    /**
     * Constructor
     *
     * @param Premium $premium
     * @param ResultFactory $resultFactory
     * @param Http $httpResponse
     */
    public function __construct(
        Premium $premium,
        ResultFactory $resultFactory,
        Http $httpResponse
    ) {
        $this->premium = $premium;
        $this->resultFactory = $resultFactory;
        $this->httpResponse = $httpResponse;
    }

    /**
     * After execute
     *
     * @param Action $subject
     * @param ResultInterface $result
     *
     * @return ResultInterface
     */
    public function afterExecute(Action $subject, $result)
    {
        $request = $subject->getRequest();
        if ($result instanceof \Magento\Framework\Controller\Result\Json &&
            $request->getModuleName() === 'wtai' &&
            $request->getRouteName() === 'wtai'
        ) {
            $result->renderResult($this->httpResponse);
            $responseData = json_decode($this->httpResponse->getContent(), true);

            try {
                $credits = [
                    'is_premium' => $this->premium->getIsPremium(),
                    'total_credits' => $this->premium->getTotalCredits(),
                    'is_free_premium_credits' => $this->premium->getFreePremiumCredits(),
                    'is_already_premium' => $this->premium->getFreeCreditsAlreadyPremium(),
                    'is_welcome_premium_20' => $this->premium->getWelcomePremium20(),
                    'is_unlocked_premium' => $this->premium->getUnlockedPremium(),
                    'is_terms_accepted' => true
                ];
            } catch (\WriteTextAI\WriteTextAI\Exception\TermsException $e) {
                $credits = [
                    'is_premium' => false,
                    'total_credits' => 0,
                    'is_free_premium_credits' => false,
                    'is_already_premium' => false,
                    'is_welcome_premium_20' => false,
                    'is_unlocked_premium' => false,
                    'is_terms_accepted' => false
                ];
            }

            $newResponse = $this->resultFactory->create(ResultFactory::TYPE_JSON);
            $newResponse->setData(array_merge($responseData, $credits));
            return $newResponse;
        }

        return $result;
    }
}
