/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'underscore',
    'Magento_Ui/js/grid/filters/filters',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products',
    'WriteTextAI_WriteTextAI/js/model/history-log',
    'WriteTextAI_WriteTextAI/js/categories/form/element/status',
    'WriteTextAI_WriteTextAI/js/utils/grid/filters'
], function (
    _,
    Filters,
    referenceProducts,
    historyLog,
    wtaiStatusComponent,
    filtersUtils
) {
    'use strict';

    return Filters.extend({
        defaults: {
            template: 'WriteTextAI_WriteTextAI/grid/filters',
            defaultStoreId: 0
        },

        /** @inheritdoc */
        initObservable: function () {
            return this._super().observe([]);
        },

        /** @inheritdoc */
        apply: function () {
            filtersUtils.removeParameters();
            wtaiStatusComponent.prototype.outerClick(this.getTextFields());
            this.filters = this.removeUndefinedStoreFilter(this.filters);
            return this._super();
        },

        /** @inheritdoc */
        clear: function (filter) {
            filtersUtils.removeParameters();
            return this._super(filter);
        },

        /**
         * Reset text fields if empty
         *
         * @param {Object} filters
         * @returns {Object}
         */
        getTextFields: function (filters) {
            var textFields = ['category page title', 'category page description', 'category description'];

            return textFields;
        },

        /**
         * Finds filters with a not empty data
         * and sets them to the 'active' filters array.
         *
         * @returns {Filters} Chainable.
         */
        updateActive: function () {
            var appliedClone = [],
                storeId = '',
                applied = [],
                textFields = 3;
            
            filtersUtils.processRecordIdsFilter(this.applied, this);
                
            /** remove writetextai_status if it is all and all fields selected */
            appliedClone = _.clone(this.applied);
            if (appliedClone.writetextai_status?.auto_text_optimizations !== undefined) {
                filtersUtils.processCustomUrlFilters(appliedClone.writetextai_status, this);
            }

            if (appliedClone.writetextai_status?.ai_fields?.length >= textFields &&
                appliedClone.writetextai_status?.status === 'all' &&
                appliedClone.writetextai_status?.auto_text_optimizations === 'show_all'
            ) {
                delete appliedClone.writetextai_status;
            }

            applied = _.keys(appliedClone);

            this.active = this.elems.filter(function (elem) {
                return _.contains(applied, elem.index);
            });
            
            storeId = this.applied.store_id || this.defaultStoreId;

            /**
             * Used for filtering reference products in bulk generate prompt,
             * uploading images in bulk generation,
             * filtering history logs in grid when closing edit page,
             * and pre selected stores in transfer prompt
             */
            referenceProducts.storeId(storeId);

            historyLog.storeId(storeId);

            return this;
        },

        /**
         * Set store filter to default.
         *
         * @param {Object} filters
         * @returns {Filters} Chainable.
         */
        removeUndefinedStoreFilter: function (filters) {
            if (filters.store_id === undefined) {
                delete filters.store_id;
            }

            return filters;
        },

        /**
         * Get parameter by name
         *
         * @param {String} name
         *
         * @returns {String}
         */
        getParameterValueByName: function (name) {
            var url = new URL(window.location.href),
                searchParams = new URLSearchParams(url.search),
                value = searchParams.get(name);

            return value ? value : '';
        }
    });
});
