/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products',
    'mage/translate',
    'mage/mage',
    'mage/tabs',
    'mage/validation',
    'jquery/ui'
], function (
    $,
    ko,
    Component,
    prompt,
    totalCredits,
    textfieldsUtils,
    referenceProducts,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            tooltipSelector: '.wtai-bulk-generate-form .admin__field-tooltip-action',
            selectedFields: [],
            referenceProductChecked: false,
            descMax: '',
            descMin: '',
            excMax: '',
            excMin: '',
            disallowedCombinations: [],
            tonesSelected: [],
            stylesSelected: '',
            audienceSelected: [],
            wordsAllowed: false,
            specialInstructions: '',
            attributesSelected: [],
            autoOptimizeKeywords: false,
            autoTextOptimizationSelected: 'flag',
            searchIntent: [],
            searchIntents: [
                'transactional',
                'commercial',
                'navigational',
                'informational'
            ],
            clusterCount: 4,
            isLoading: true,
        },

        isSpinning: false,
        typingTimer: null,
        isReopening: false,
        isRestoring: false,
        
        /** @inheritdoc */
        initialize: function () {
            this._super();

            $('.wtai-bulk-generate-form-container > *:not(.wtai-reference-product, .wtai-reference-product *)').applyBindings(this);

            self = this;

            self.attributesSelected.subscribe(function (attributesSelected) {
                if (!self.isReopening) {
                    prompt.attributesSelected([]);
                    prompt.attributesSelected(attributesSelected);
                    self.savePreferences('attributes_selected', attributesSelected);
                }
            });

            self.specialInstructions.subscribe(function (specialInstructions) {
                prompt.specialInstructions(specialInstructions);
            });

            self.selectedFields.subscribe(function (selectedFields) {
                prompt.selectedFields(selectedFields);
                if (!self.isReopening) {
                    self.savePreferences('generate', selectedFields);
                }
            });

            self.tonesSelected.subscribe(function () {
                prompt.tonesSelected(self.tonesSelected());
                if (!self.isReopening) {
                    self.savePreferences('tones', self.tonesSelected());
                }
            });

            self.stylesSelected.subscribe(function (stylesSelected) {
                prompt.stylesSelected(stylesSelected);
                self.savePreferences('style', stylesSelected);
            });

            self.audienceSelected.subscribe(function () {
                prompt.audienceSelected(self.audienceSelected());
                if (!self.isReopening) {
                    self.savePreferences('audiences', self.audienceSelected());
                }
            });

            self.descMin.subscribe(function (descMin) {
                prompt.descMin(descMin);
                if (!self.isSpinning) {
                    self.savePreferences('desc_min', descMin);
                }
            });

            self.descMax.subscribe(function (descMax) {
                prompt.descMax(descMax);
                if (!self.isSpinning) {
                    self.savePreferences('desc_max', descMax);
                }
            });

            self.excMax.subscribe(function (excMax) {
                prompt.excMax(excMax);
                if (!self.isSpinning) {
                    self.savePreferences('exc_max', excMax);
                }
            });

            self.excMin.subscribe(function (excMin) {
                prompt.excMin(excMin);
                if (!self.isSpinning) {
                    self.savePreferences('exc_min', excMin);
                }
            });

            self.referenceProductChecked.subscribe(
                function (referenceProductChecked) {
                    prompt.referenceProductChecked(referenceProductChecked);
                    self.savePreferences('reference_checked', referenceProductChecked);
                }
            );

            self.autoOptimizeKeywords.subscribe(function (autoOptimizeKeywords) {
                prompt.autoOptimizeKeywords(autoOptimizeKeywords);
                self.savePreferences('auto_optimize_keywords', autoOptimizeKeywords);
            });

            self.autoTextOptimizationSelected.subscribe(function (autoTextOptimizationSelected) {
                prompt.autoTextOptimizationSelected(autoTextOptimizationSelected);
                self.savePreferences('automatic_text_optimization', autoTextOptimizationSelected);
            });

            self.searchIntent.subscribe(function (searchIntent) {
                prompt.searchIntent(searchIntent);
                if (!self.isReopening) {
                    self.savePreferences('search_intent', searchIntent.join(','));
                }
            });

            self.clusterCount.subscribe(function (clusterCount) {
                prompt.clusterCount(clusterCount);
                if (!self.isSpinning) {
                    self.savePreferences('cluster_count', clusterCount);
                }
            });

            referenceProducts.referenceProductSelected.subscribe(function (referenceProductSelected) {
                if (referenceProducts.initialized()) {
                    self.savePreferences('reference_selected', referenceProductSelected);
                }
            });

            self.showLoader();
            self.initBulkTabs();
            self.initTooltip();
            self.initValidations();
            self.initTriggerUpdate();
            self.initSpinner();
            self.initPreferences();
            self.removeLoader();
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'selectedFields',
                'referenceProductChecked',
                'descMax',
                'descMin',
                'excMax',
                'excMin',
                'disallowedCombinations',
                'tonesSelected',
                'stylesSelected',
                'audienceSelected',
                'specialInstructions',
                'attributesSelected',
                'autoOptimizeKeywords',
                'autoTextOptimizationSelected',
                'searchIntent',
                'clusterCount',
                'isLoading'
            ]);
            
            this.allFieldsSelected = ko.computed({
                read: function () {
                    const selectedFields = this.selectedFields() || [];
                    const allFields = [...this.allFields] || [];
                    return selectedFields.length === allFields.length;
                }.bind(this),
                write: function (value) {
                    const allFields = [...this.allFields ]|| [];
                    const selectedFields = value ? allFields : [];
                    this.selectedFields(selectedFields);
                }.bind(this)
            }, this);
            
            this.allTonesSelected = ko.computed({
                read: function () {
                    const tones = this.tonesSelected() || [];
                    const allTones = [...this.tones] || [];
                    return tones.length === allTones.length;
                }.bind(this),
                write: function (value) {
                    const allTones = [...this.tones ]|| [];
                    const tones = value ? allTones : [];
                    this.tonesSelected(tones);
                }.bind(this)
            }, this);
            
            this.allAudienceSelected = ko.computed({
                read: function () {
                    const audience = this.audienceSelected() || [];
                    const allAudience = [...this.audience] || [];
                    return audience.length === allAudience.length;
                }.bind(this),
                write: function (value) {
                    const allAudience = [...this.audience ]|| [];
                    const audience = value ? allAudience : [];
                    this.audienceSelected(audience);
                }.bind(this)
            }, this);
            
            this.allAttributesSelected = ko.computed({
                read: function () {
                    const attributes = this.attributesSelected() || [];
                    const allAttributes = [...this.attributes] || [];
                    return attributes.length === allAttributes.length;
                }.bind(this),
                write: function (value) {
                    const allAttributes = [...this.attributes ]|| [];
                    const attributes = value ? allAttributes : [];
                    this.attributesSelected(attributes);
                }.bind(this)
            }, this);

            this.submitAction = ko.computed(function () {
                return prompt.submitAction();
            });

            this.rewriteEnabled = ko.computed(function () {
                return !this.referenceProductChecked();
            }, this);

            this.isPremium = ko.computed(function () {
                return totalCredits.isPremium();
            });

            this.getOtherProductDetailsLength = ko.computed(function () {
                return this.specialInstructions().length + '/' + this.maxOtherDetailsLength + ' ' +  $t('Char');
            }, this);

            this.searchIntentCommadSeparated = ko.computed(function () {
                return this.searchIntent().join(', ');
            }, this);

            this.referenceProductEnabled = ko.computed(function () {
                return prompt.submitAction() === 'generate';
            }, this);

            return this;
        },

        /**
         * Initialize preferences
         *
         * Set model values on page load.
         * Model values updates on change.
         * Retain selected fields when reopening the modal.
         */
        initPreferences: function () {
            if (!prompt.keywordAnalysisPromptInitialized()) {
                prompt.autoTextOptimizationSelected(this.autoTextOptimizationSelected());
                prompt.searchIntent(this.searchIntent());
                prompt.clusterCount(this.clusterCount());
                prompt.keywordAnalysisPromptInitialized(true);
            } else {
                this.isReopening = true;
                this.autoTextOptimizationSelected(prompt.autoTextOptimizationSelected());
                this.searchIntent(prompt.searchIntent());
                this.clusterCount(prompt.clusterCount());
                this.isReopening = false;
            }
            
            if (!prompt.initialized()) {
                prompt.descMin(self.descMin());
                prompt.descMax(self.descMax());
                prompt.excMin(self.excMin());
                prompt.excMax(self.excMax());
                prompt.selectedFields(self.selectedFields());
                prompt.tonesSelected(self.tonesSelected());
                prompt.stylesSelected(self.stylesSelected());
                prompt.audienceSelected(self.audienceSelected());
                prompt.specialInstructions(self.specialInstructions());
                prompt.attributesSelected([]);
                prompt.attributesSelected(self.attributesSelected());
                prompt.autoOptimizeKeywords(self.autoOptimizeKeywords());
                prompt.referenceProductChecked(self.referenceProductChecked());
                prompt.initialized(true);
            } else {
                self.isReopening = true;
                self.descMin(prompt.descMin());
                self.descMax(prompt.descMax());
                self.excMin(prompt.excMin());
                self.excMax(prompt.excMax());
                self.selectedFields(prompt.selectedFields());
                self.tonesSelected(prompt.tonesSelected());
                self.stylesSelected(prompt.stylesSelected());
                self.audienceSelected(prompt.audienceSelected());
                self.specialInstructions(prompt.specialInstructions());
                self.attributesSelected([]);
                self.attributesSelected(prompt.attributesSelected());
                self.autoOptimizeKeywords(prompt.autoOptimizeKeywords());
                self.referenceProductChecked(prompt.referenceProductChecked());
                self.isReopening = false;
            }
        },

        /**
         * Initialize update trigger
         */
        initTriggerUpdate: function () {
            $('#wtaiPrompt').trigger('contentUpdated');
        },
        
        /**
         * Show loader
         */
        showLoader: function () {
            self.isLoading(true);
        },

        /**
         * Remove loader
         */
        removeLoader: function () {
            setTimeout(function () {
                self.isLoading(false);
            }, 100);
        },

        /**
         * Restore user preferences
         */
        restore: function () {
            self.isRestoring = true;
            self.descMin(self.globalSettings.desc_min);
            self.descMax(self.globalSettings.desc_max);
            self.excMin(self.globalSettings.exc_min);
            self.excMax(self.globalSettings.exc_max);
            self.selectedFields(self.globalSettings.fields);
            self.tonesSelected(self.globalSettings.tones);
            self.stylesSelected(self.globalSettings.styles);
            self.audienceSelected(self.globalSettings.audience);
            self.referenceProductChecked(false);
            self.specialInstructions(self.globalSettings.special_instructions);
            self.attributesSelected([]);
            self.attributesSelected(self.globalSettings.attributes);
            self.autoOptimizeKeywords(self.globalSettings.auto_optimize_keywords);
            self.autoTextOptimizationSelected(self.globalSettings.automatic_text_optimization);
            self.searchIntent([...self.globalSettings.search_intent]);
            self.clusterCount(self.globalSettings.cluster_count);
            self.referenceProductChecked(false);
            referenceProducts.referenceProductSelected('');
            self.isRestoring = false;
            $.ajax({
                url: self.restoreUrl,
                type: 'GET',
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Save user preferences
         *
         * @param {String} field
         * @param {String} selected
         *
         * @return {void}
         */
        savePreferences: function (field, selected) {
            var value = selected;
            
            if (value === null || value.length <= 0) {
                value = '';
            }

            if (self.isRestoring) {
                return;
            }

            $.ajax({
                url: self.saveUrl,
                type: 'POST',
                data: {
                    [field]: value,
                    entity_type: 'product',
                    scope: 'grid'
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Validate number.
         *
         * @param {Object} data
         * @param {Object} event
         *
         * @return {Boolean}
         */
        ensureNumerical: function (data, event) {
            return textfieldsUtils.ensureNumerical(event.key);
        },

        /**
         * Is intent selected
         */
        isIntentSelected: function (intent) {
            return self.searchIntent().indexOf(intent) !== -1;
        },

        /**
         * Select search intent
         *
         * @param {String} intent
         * @return {void}
         */
        selectIntent: function (intent) {
            var searchIntent = self.searchIntent();
            if (searchIntent.length <= 1 &&
                searchIntent.indexOf(intent) !== -1
            ) {
                return;
            }

            if (searchIntent.indexOf(intent) === -1) {
                searchIntent.push(intent);
            } else {
                searchIntent = _.without(searchIntent, intent);
            }
            self.searchIntent(searchIntent);
        },

        /**
         * Handle input special instructions.
         */
        handleInputSpecialInstructions: function () {
            clearTimeout(self.typingTimer);

            self.typingTimer = setTimeout(function () {
                self.savePreferences('special_instructions', self.specialInstructions());
            }, 2000);

            return true;
        },

        /**
         * Initialize spinner
         */
        initSpinner: function () {
            setTimeout(
                function () {
                    $('.wtai-bulk-generate-popup .wtai-cluster-count').spinner({
                        numberFormat: 'n',
                        min: this.minClusterCount,
                        max: this.maxClusterCount,
                        spin: function (event, ui) {
                            self.isSpinning = true;
                            self.clusterCount(ui.value);
                            self.isSpinning = false;
                        },
                        change: function (event) {
                            self.clusterCount(event.target.value);
                        }
                    });
                    $('.wtai-bulk-generate-popup .wtai-input-number').spinner({
                        numberFormat: 'n',
                        min: this.minOutputWords,
                        max: this.maxOutputWords,
                        disabled: !self.wordsAllowed,
                        spin: function (event, ui) {
                            var value = ui.value,
                                name = $(event.target).attr('name');
                            
                            self.isSpinning = true;
                            switch (name) {
                                case 'description_min':
                                    self.descMin(value);
                                    break;
                                case 'description_max':
                                    self.descMax(value);
                                    break;
                                case 'excerpt_min':
                                    self.excMin(value);
                                    break;
                                case 'excerpt_max':
                                    self.excMax(value);
                                    break;
                            }
                            self.isSpinning = false;
                        },
                        change: function (event) {
                            var value = event.target.value,
                                name = $(event.target).attr('name');
                            
                            switch (name) {
                                case 'description_min':
                                    self.descMin(value);
                                    break;
                                case 'description_max':
                                    self.descMax(value);
                                    break;
                                case 'excerpt_min':
                                    self.excMin(value);
                                    break;
                                case 'excerpt_max':
                                    self.excMax(value);
                                    break;
                            }
                        }
                    });
                }.bind(this),
                100
            );
        },

        /**
         * Initialize tooltip
         */
        initTooltip: function () {
            $(self.tooltipSelector).dropdown({
                parent: '.admin__field-tooltip',
                autoclose: true
            });
        },

        /**
         * Initialize validations
         */
        initValidations: function () {
            $('#wtaiBulkGenerateForm').mage('validation', {
                rules: {
                    'fields[]': {
                        required: true
                    },
                    'tones[]': {
                        required: {
                            param: true
                        }
                    },
                    styles: {
                        required: {
                            param: true
                        }
                    },
                    wtaiGridReferenceProduct: {
                        required: {
                            param: true,
                            depends: function () {
                                return $('#wtaiGridReferenceProductChecked').is(
                                    ':checked'
                                );
                            }
                        }
                    }
                },
                messages: {
                    'fields[]': {
                        required: $t('Please select at least one text type.')
                    },
                    'tones[]': {
                        required: $t('Please select at least one tone.')
                    },
                    styles: {
                        required: $t('Please select at least one style.')
                    },
                    wtaiGridReferenceProduct: {
                        required: $t('Please select a reference product.')
                    }
                },
                errorPlacement: function (error) {
                    $('.wtai-tabs-error-message').html(error);
                }
            });
        },

        /**
         * Initialize bulk tabs
         */
        initBulkTabs: function () {
            setTimeout(function () {
                $('.wtai-tabs').tabs({
                    active: 0
                });
            }, 100);
        },

        /**
         * Check if combination is not allowed
         *
         * @param {String} id
         * @param {String} type
         *
         * @return {Boolean}
         */
        isCombinationDisallowed: function (id, type) {
            var result,
                isStyleChecked = false,
                isToneChecked = false,
                isAudienceChecked = false,
                combinationHasAudience = false;

            self = this;

            try {
                /* Check if combination is available */
                result = self
                    .disallowedCombinations()
                    .find(function (combination) {
                        return combination.combination.some(function (obj) {
                            return obj.type === type && obj.id === id;
                        });
                    });

                if (!(self.disallowedCombinations() && result)) {
                    return false;
                }

                switch (type) {
                    case 'Tone':
                        isStyleChecked = result.combination.some(
                            function (obj) {
                                return (
                                    obj.type === 'Style' &&
                                    self.stylesSelected() === obj.id
                                );
                            }
                        );
                        combinationHasAudience =
                            result.combination.some(function (obj) {
                                return obj.type === 'Audience';
                            });

                        if (combinationHasAudience) {
                            isAudienceChecked =
                                self
                                    .audienceSelected()
                                    .indexOf(combinationHasAudience.id) !==
                                -1;

                            return isStyleChecked && isAudienceChecked;
                        }
                        return isStyleChecked;

                    case 'Style':
                        isToneChecked = result.combination.some(
                            function (obj) {
                                return (
                                    obj.type === 'Tone' &&
                                    self.tonesSelected().indexOf(obj.id) !==
                                        -1
                                );
                            }
                        );
                        combinationHasAudience =
                            result.combination.some(function (obj) {
                                return obj.type === 'Audience';
                            });

                        if (combinationHasAudience) {
                            isAudienceChecked =
                                self
                                    .audienceSelected()
                                    .indexOf(combinationHasAudience.id) !==
                                -1;

                            return isToneChecked && isAudienceChecked;
                        }
                        return isToneChecked;

                    case 'Audience':
                        isToneChecked = result.combination.some(
                            function (obj) {
                                return (
                                    obj.type === 'Tone' &&
                                    self.tonesSelected().indexOf(obj.id) !==
                                        -1
                                );
                            }
                        );
                        isStyleChecked = result.combination.some(
                            function (obj) {
                                return (
                                    obj.type === 'Style' &&
                                    self.stylesSelected() === obj.id
                                );
                            }
                        );

                        return isStyleChecked && isToneChecked;

                    default:
                        return false;
                }
            } catch (e) {
                return false;
            }
        },

        /**
         * Get tooltip text based on disallowed combination
         *
         * @param {String} id
         * @param {String} type
         *
         * @return {String}
         */
        getTooltipCombinationDisallowedText: function (id, type) {
            var result,
                isStyleChecked = false,
                isToneChecked = false,
                textBase = '';

            self = this;

            try {
                /* Check if combination is available */
                result = self
                    .disallowedCombinations()
                    .find(function (combination) {
                        return combination.combination.some(function (obj) {
                            return obj.type === type && obj.id === id;
                        });
                    });

                if (!result) {
                    return '';
                }

                switch (type) {
                    case 'Tone':
                        isStyleChecked = result.combination.some(
                            function (obj) {
                                if (obj.type === 'Style' && self.stylesSelected() === obj.id) {
                                    textBase = self.processToolTip(obj.id, 'Style');
                                    return true;
                                }
                                return false;
                            }
                        );
                        return textBase;

                    case 'Style':
                        isToneChecked = result.combination.some(
                            function (obj) {
                                if ( obj.type === 'Tone' &&
                                    self.tonesSelected().indexOf(obj.id) !== -1) {
                                    textBase = self.processToolTip(obj.id, 'Tone');
                                    return true;
                                }
                                return false;
                            }
                        );
                        return textBase;

                    default:
                        return '';
                }
            } catch (e) {
                return '';
            }
        },

        /**
         * Process tooltip content
         *
         * @param {String} selector
         * @param {String} stringValue
         *
         * @return {String}
         */
        processToolTip: function (stringValue, type) {
            var textBase = '';
            switch (type) {
                case 'Tone':
                    textBase = $t("Unavailable when the '%1' tone is selected.");
                    break;
                case 'Style':
                    textBase = $t("Unavailable when the '%1' style is selected.");
                    break;
                default:
                    textBase = $t("Unavailable when the '%1' audience is selected.");
                    break;
            }
            return $t(textBase).replace('%1', stringValue);
        },

        /**
         * Validate min and max fields.
         *
         * @param {Object} data
         * @param {Object} event
         *
         * @return {void}
         */
        validateMinMax: function (data, event) {
            var target = event.target,
                value = Number(target.value),
                min = Number(target.min),
                max = Number(target.max),
                name = target.name;

            const mapping = {
                description_min: self.descMin,
                description_max: self.descMax,
                excerpt_min: self.excMin,
                excerpt_max: self.excMax,
                cluster_count: self.clusterCount
            };

            if (value < min || value > max) {
                const func = mapping[name],
                    val = value < min ? min : max;

                $(target).val(val);
                func(val);
            }
        },
    });
});
