/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt'
], function (
    $,
    ko,
    Component,
    prompt
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            submitAction: 'generate'
        },

        isReopening: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            $('#wtaiPromptFooter > *').applyBindings(this);

            self = this;

            self.submitAction.subscribe(function (submitAction) {
                prompt.submitAction(submitAction);
            });

            self.initPreferences();
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'submitAction'
            ]);
            
            this.rewriteEnabled = ko.computed(function () {
                return !prompt.referenceProductChecked();
            });

            return this;
        },

        /**
         * Initialize preferences
         *
         * Set model values on page load.
         * Model values updates on change.
         * Retain selected fields when reopening the modal.
         */
        initPreferences: function () {
            if (!prompt.footerInitialized()) {
                prompt.submitAction(self.submitAction());
                prompt.footerInitialized(true);
            } else {
                self.isReopening = true;
                self.submitAction(prompt.submitAction());
                self.isReopening = false;
            }
        },
    });
});
