/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'Magento_Ui/js/form/element/ui-select',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'underscore',
    'mage/translate',
    'Magento_Ui/js/block-loader',
    'Magento_Ui/js/lib/key-codes'
], function (
    $,
    Select,
    referenceProducts,
    prompt,
    _,
    $t,
    blockLoader,
    keyCodes
) {
    'use strict';

    /**
     * @var {Select} self
     */
    var self;

    try {
        blockLoader();
    } catch (e) {
        console.log("blockloader failed", e);
    }

    return Select.extend({
        defaults: {
            filterInputSelector:
                '.wtai-bulk-generate-form .wtai-reference-product .admin__action-multiselect-search',
            template: 'ui/form/field',
            multiple: false,
            filterOptions: true,
            selectedPlaceholders: {
                defaultPlaceholder: $t('Select or enter product name')
            },
            filterPlaceholder: $t('Search products'),
            labelVisible: false,
            referenceProductChecked: false,
            storeId: 0,
            selectedOptionLabel: '',
            page: 1,
            pageSize: 50,
            totalSize: 0,
            lazyLoad: true,
            isLoading: false,
            listens: {
                filterInputValue: ''
            }
        },

        typingTimer: null,
        isOptionsChanged: false,
        productsLoaded: false,
        isValueChanged: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            $('.wtai-bulk-generate-form .wtai-reference-product > *').applyBindings(this);

            self = this;

            this.initStoreId();

            self.options.subscribe(function () {
                if (!self.isOptionsChanged) {
                    self.resetOptions();
                }
            });

            self.totalSize.subscribe(function (totalSize) {
                self.itemsQuantity(totalSize + ' ' + self.quantityPlaceholder);
            });

            self.storeId.subscribe(function () {
                self.filterOptionsList(false, true, false);
            });

            prompt.referenceProductChecked.subscribe(
                function (referenceProductChecked) {
                    self.referenceProductChecked(referenceProductChecked);
                    if (referenceProductChecked && !self.productsLoaded) {
                        self.productsLoaded = true;
                    }
                }
            );

            self.referenceProductChecked.subscribe(function (referenceProductChecked) {
                if (!referenceProductChecked) {
                    this.value(null);
                }
            }, this);

            referenceProducts.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            this.value.subscribe(function (value) {
                if (!self.isValueChanged) {
                    referenceProducts.referenceProductSelected(value);
                }
            });
            
            referenceProducts.referenceProductSelected.subscribe(function (referenceProductSelected) {
                self.isValueChanged = true;
                self.value(referenceProductSelected);
                self.isValueChanged = false;
            });

            this.initPreferences();
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'referenceProductChecked',
                'storeId',
                'selectedOptionLabel',
                'page',
                'pageSize',
                'totalSize',
                'isLoading'
            ]);

            return this;
        },

        /**
         * Initialize preferences
         *
         * Set model values on page load.
         * Model values updates on change.
         * Retain selected fields when reopening the modal.
         */
        initPreferences: function () {
            self.referenceProductChecked(prompt.referenceProductChecked());
            self.filterOptionsList(false, true, false); /** Load products list */
            
            if (!referenceProducts.initialized()) {
                referenceProducts.referenceProductSelected(self.value());
                referenceProducts.initialized(true);
            } else {
                self.isValueChanged = true;
                self.value(referenceProducts.referenceProductSelected());
                self.isValueChanged = false;
            }
        },

        /**
         * Init store id
         */
        initStoreId: function () {
            self.storeId(referenceProducts.storeId());
        },

        /**
         * Reset options and filter input value
         */
        resetOptions: function () {
            self.page(1);
            self.filterInputValue('');
        },

        /**
         * Load more options on scroll down
         *
         * @param {Object} data
         * @param {Event} event
         *
         * @returns {void}
         */
        onScrollDown: function (data, event) {
            var clientHight =
                    event.target.scrollTop + event.target.clientHeight,
                scrollHeight = event.target.scrollHeight;

            if (this.totalSize() <= this.options().length && !this.lazyLoad) {
                return;
            }

            if (clientHight > scrollHeight - this.deviation) {
                self.page(self.page() + 1);
                self.filterOptionsAction(true);
            }
        },

        /** @inheritdoc */
        keyDownHandlers: function () {
            return {
                escapeKey: this.escapeKeyHandler,
                spaceKey: this.enterKeyHandler,
                pageUpKey: this.pageUpKeyHandler,
                pageDownKey: this.pageDownKeyHandler
            };
        },
        
        /** @inheritdoc */
        filterOptionsKeydown: function (data, event) {
            var key = keyCodes[event.keyCode];

            if (key === 'enterKey') {
                self.filterOptionsList(false, false, false);
            }
            
            return this._super(data, event);
        },

        /**
         * Filtered options list by value from filter options list
         *
         * @param {Boolean} showMore
         * @param {Boolean} clear
         * @param {Boolean} delay
         *
         * @returns {void}
         */
        filterOptionsList: function (showMore = false, clear = false, delay = true) {
            clearTimeout(self.typingTimer);
            if (delay) {
                self.typingTimer = setTimeout(function () {
                    self.filterOptionsAction(showMore, clear);
                }, 2000);
            } else {
                self.filterOptionsAction(showMore, clear);
            }
        },

        /**
         * Filtered options list by value from filter options list
         *
         * @param {Boolean} showMore
         * @param {Boolean} clear
         *
         * @returns {void}
         */
        filterOptionsAction: function (showMore, clear) {
            var url = self.filterReferencesUrl,
                storeId = self.storeId(),
                page,
                pageSize = self.pageSize(),
                search = self.filterInputValue().trim().toLowerCase();

            self.isLoading(true);
            
            $('.wtai-bulk-generate-popup.wtai-confirm .action-accept').prop('disabled', true);

            if (!showMore) {
                self.page(1);
            }

            page = self.page();

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    search: search,
                    store_id: storeId,
                    page: page,
                    page_size: pageSize
                },
                dataType: 'json',
                success: function (response) {
                    if (response.success) {
                        if (clear) {
                            self.options(response.products);
                        } else {
                            self.isOptionsChanged = true;
                            self.options(response.products);
                            self.isOptionsChanged = false;
                        }

                        self.totalSize(response.total);
                        self.itemsQuantity(
                            response.total + ' ' + self.quantityPlaceholder
                        );
                        setTimeout(function () {
                            $(self.filterInputSelector).trigger('focus');
                        });
                    }
                }
            }).always(function () {
                self.isLoading(false);
                $('.wtai-bulk-generate-popup.wtai-confirm .action-accept').prop('disabled', false);
            });
        },

        /**
         * Set caption
         */
        setCaption: function () {
            var length,
                caption = '';

            if (!_.isArray(this.value()) && this.value()) {
                length = 1;
            } else if (this.value()) {
                length = this.value().length;
            } else {
                this.value([]);
                length = 0;
            }
            this.warn(caption);

            if (
                this.isDisplayMissingValuePlaceholder &&
                length &&
                !this.getSelected().length
            ) {
                caption = this.missingValuePlaceholder.replace(
                    '%s',
                    this.value()
                );
                this.placeholder(this.selectedOptionLabel());
                this.warn(caption);

                return this.placeholder();
            }

            if (length > 1) {
                this.placeholder(
                    length + ' ' + this.selectedPlaceholders.lotPlaceholders
                );
            } else if (length && this.getSelected().length) {
                this.placeholder(this.getSelected()[0].label);
            } else {
                this.placeholder(this.selectedPlaceholders.defaultPlaceholder);
            }

            return this.placeholder();
        },

        /**
         * Show more options
         */
        showMore: function () {
            self.page(self.page() + 1);
            self.filterOptionsList(true, false, false);
        },

        /**
         * Toggle activity list element
         *
         * @param {Object} data - selected option data
         * @returns {Object} Chainable
         */
        toggleOptionSelected: function (data) {
            self.selectedOptionLabel(data.label);
            prompt.selectedReferenceProduct(data);
            return this._super(data);
        },

        /**
         * Get path to current option
         *
         * @param {Object} data - option data
         * @returns {String} path
         */
        getPath: function (data) {
            if (typeof data.path !== 'undefined') {
                return this._super(data);
            }
        }
    });
});
