/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'Magento_Ui/js/grid/provider',
    'WriteTextAI_WriteTextAI/js/utils/grid/filters'
], function (
    Provider,
    filtersUtils
) {
    'use strict';

    return Provider.extend({
        defaults: {
            isLoading: false
        },

        /**
         * @inheritdoc
         */
        reload: function (options) {
            var custom,
                showLoader,
                debounce,
                request;

            options = options || {};

            custom =
                typeof options.custom !== 'undefined'
                    ? options.custom
                    : false;
            showLoader =
                typeof options.showLoader !== 'undefined'
                    ? options.showLoader
                    : true;
            debounce =
                typeof options.debounce !== 'undefined'
                    ? options.debounce
                    : false;

            if (debounce && this.isLoading) {
                return;
            }

            this.isLoading = true;

            if (this.params.filters) {
                if (this.params.filters['writetextai_status']) {
                    filtersUtils.processCustomUrlFilters(this.params.filters['writetextai_status'], this);
                }
                
                filtersUtils.processRecordIdsFilter(this.params.filters, this);
            }
            
            request = this.storage().getData(this.params, options);

            if (custom) {
                if (showLoader) {
                    this.trigger('reload');
                }
            } else {
                this.trigger('reload');
            }

            request.done(this.onReload).fail(this.onError.bind(this));

            return request;
        },

        /**
         * @inheritdoc
         */
        onReload: function (data) {
            this._super(data);

            this.isLoading = false;
        },

        /**
         * Get parameter by name
         *
         * @param {String} name
         *
         * @returns {String}
         */
        getParameterValueByName: function (name) {
            var url = new URL(window.location.href),
                searchParams = new URLSearchParams(url.search),
                value = searchParams.get(name);

            return value ? value : '';
        }
    });
});
