/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'Magento_Ui/js/modal/modal'
], function ($, ko, Component, totalCredits) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            showPopup: false
        },
    
        /** @inheritdoc */
        initialize: function () {
            this._super();
    
            self = this;

            totalCredits.isFreeCreditsAlreadyPremium.subscribe(function (isFreeCreditsAlreadyPremium) {
                self.showPopup(isFreeCreditsAlreadyPremium);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'showPopup'
            ]);

            return this;
        },

        /**
         * Set free premium credits to false
         *
         * @returns {void}
         */
        setFreePremiumCreditsToFalse: function () {
            totalCredits.isFreeCreditsAlreadyPremium(false);
            
            $.ajax({
                url: self.setToFalseUrl,
                type: 'POST',
                data: {
                    'is_free_premium_credits': false
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Close modal
         */
        closeModal: function () {
            self.setFreePremiumCreditsToFalse();
        }
    });
});
