/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'mage/translate',
    'Magento_Ui/js/modal/modal'
], function ($, Component, $t) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            selectors: {
                modal: '.wtai-update-to-v21-popup-modal',
                overlay: '.wtai-update-to-v21-popup-overlay',
            },
            classes: {
                modal: 'wtai-update-to-v21-popup-modal',
                overlay: 'wtai-update-to-v21-popup-overlay'
            }
        },

        popup: null,

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([]);
    
            return this;
        },
    
        /** @inheritdoc */
        initialize: function () {
            this._super();
    
            self = this;
    
            this.initModal();
        },

        /**
         * Initialize history log modal.
         */
        initModal: function () {
            this.popup = $(this.selectors.modal).modal({
                innerScroll: true,
                modalClass: self.classes.modal,
                overlayClass: self.classes.overlay,
                outerClickHandler: function () {
                    $(this.selectors.modal).modal('closeModal');
                }.bind(this),
                buttons: [],
                closed: function () {
                    $(self.selectors.overlay).remove();
                    self.save();
                }
            });

            $(this.selectors.modal).css('display', 'block');

            if (this.showPopup) {
                this.popup.modal('openModal');
            }
        },

        /**
         * Save guide steps checkbox
         */
        save: function () {
            var url = self.saveUrl;

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    type: 'update_to_v21'
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response.message);
                    }
                }
            });
        }
    });
});
