/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/model/signalr',
    'WriteTextAI_WriteTextAI/js/model/edit',
    'WriteTextAI_WriteTextAI/js/model/edit/gallery',
    'WriteTextAI_WriteTextAI/js/model/edit/feedback',
    'WriteTextAI_WriteTextAI/js/model/edit/mark-reviewed',
    'WriteTextAI_WriteTextAI/js/model/grid/listing',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/keyword-analysis',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/keywords',
    'WriteTextAI_WriteTextAI/js/model/grid/popups/keyword-analysis-progress',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/currently-generating',
    'WriteTextAI_WriteTextAI/js/grid/reload',
    'WriteTextAI_WriteTextAI/js/utils/signalr',
    'wtaiSignalR',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    textfields,
    product,
    notifications,
    signalRModel,
    editData,
    gallery,
    feedbackData,
    markReviewed,
    listingModel,
    keywordAnalysis,
    keywords,
    keywordAnalysisProgressModel,
    currentlyGeneratingModel,
    reloadGrid,
    signalrUtils,
    signalR,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            productId: '',
            storeId: '',
            connection: null,
            retryReconnectAttempt: 0,
            enableMaxRetryAttempt: false,
            maxRetryAttempt: 10,
            accessToken: '',
            signalRUrl: '',
            singleGenerate: 0,
            bulkGenerate: 1,
            fieldStatus: 2,
            optimization: 4,
            snapshot: 5,
            openAnalysis: 6,
            message: $t('Text generation completed'),
            showMessage: false,
            messageSuccess: false,
            successMessage: $t('Text generation completed'),
            pageTitle: '',
            pageDescription: '',
            productDescription: '',
            excerpt: '',
            openGraph: '',
            errorMessage: '',
            statusFailed: 'Failed',
            statusCompleted: 'Completed'
        },

        isGettingNewToken: false,
        gettingData: [],
        singleRequestChecker: [],
        latestMessageEntry: [],

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.connectTostream();

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            signalRModel.showMessage.subscribe(function (show) {
                self.showMessage(show);
            });

            signalRModel.message.subscribe(function (message) {
                self.message(message);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'productId',
                'storeId',
                'accessToken',
                'signalRUrl',
                'message',
                'showMessage',
                'messageSuccess',
                'errorMessage',
                'latestMessageEntry'
            ]);

            this.generateComplete = ko.computed(function () {
                var fieldsGenerated = signalRModel.fieldsGenerated(),
                    fieldsValues = Object.values(fieldsGenerated),
                    hasTrue = fieldsValues.includes(true),
                    hasFalse = fieldsValues.includes(false),
                    imagesGenerating = signalRModel.imagesGenerating(),
                    imagesGenerated = signalRModel.imagesGenerated(),
                    imagesGeneratedLength = imagesGenerated ? imagesGenerated.length : 0,
                    imagesGeneratingLength = imagesGenerating ? imagesGenerating.length : 0;

                var isGenerating = signalRModel.generating(),
                    isTextsGenerating = hasTrue || hasFalse,
                    isTextsGenerated = hasTrue && !hasFalse,
                    isImagesGenerating = imagesGeneratingLength > 0 && imagesGeneratedLength > 0,
                    isImagesGenerated = imagesGeneratingLength === imagesGeneratedLength;

                if (isGenerating && (
                    (isTextsGenerating && !isImagesGenerating && isTextsGenerated) || /** texts only */
                    (!isTextsGenerating && isImagesGenerating && isImagesGenerated) || /** images only */
                    (isTextsGenerating && isImagesGenerating && isTextsGenerated && isImagesGenerated) /** texts and images */
                )) {
                    return true;
                }

                return false;
            });

            this.generateComplete.subscribe(function (generateComplete) {
                if (generateComplete) {
                    self.catchFieldsError();
                    self.getGenerated();
                }
            });

            return this;
        },

        /**
         * Catch fields error.
         */
        catchFieldsError: function () {
            var fieldsGenerated = signalRModel.fieldsGenerated(),
                fieldsValues = Object.values(fieldsGenerated),
                hasTrue = fieldsValues.includes(true),
                hasFalse = fieldsValues.includes(false),
                imagesGenerating = signalRModel.imagesGenerating(),
                imagesGenerated = signalRModel.imagesGenerated(),
                imagesGeneratedLength = imagesGenerated ? imagesGenerated.length : 0,
                imagesGeneratingLength = imagesGenerating ? imagesGenerating.length : 0;

            var isTextsGenerating = hasTrue || hasFalse,
                isImagesGenerating = imagesGeneratingLength > 0 && imagesGeneratedLength > 0;

            self.errorMessage('');
            self.messageSuccess(true);
                
            switch (true) {
                case !isTextsGenerating && isImagesGenerating:
                    self.getAltTextsError();
                    break;
                case isTextsGenerating && !isImagesGenerating:
                    self.getNormalTextsError();
                    break;
                case isTextsGenerating && isImagesGenerating:
                    self.getAllTextsError();
                    break;
                default:
                    break;
            }
        },

        /**
         * Get all texts error.
         */
        getAllTextsError: function () {
            var errorFields = signalRModel.errorFields(),
                imagesHtml = '';

            if (gallery.failedImagesWithoutThumbnail().length > 0) {
                errorFields.push('alt text');
                gallery.failedImagesWithoutThumbnail().forEach(element => {
                    imagesHtml += '<img src="' + element + '">';
                });
            }
            
            var htmlMessage = $t('Error encountered while generating the following text: %s')
                .replace('%s', '<ul>' + errorFields.map(field => `<li>${field}</li>`).join('') + '</ul>');

            htmlMessage += imagesHtml;
            
            if (gallery.failedImagesWithoutThumbnail().length > 0 || errorFields.length > 0) {
                signalRModel.message(self.successMessage);
                self.errorMessage(htmlMessage);
            }
        },

        /**
         * Get alt texts error.
         */
        getAltTextsError: function () {
            var errorFields = ['alt text'],
                imagesHtml = $t('Error encountered while generating the following text: %s')
                    .replace('%s', '<ul>' + errorFields.map(field => `<li>${field}</li>`).join('') + '</ul>');
            
            if (gallery.failedImagesWithoutThumbnail().length > 0) {
                signalRModel.message(self.successMessage);
                gallery.failedImagesWithoutThumbnail().forEach(element => {
                    imagesHtml += '<img src="' + element + '">';
                });
                self.errorMessage(imagesHtml);
            }
        },

        /**
         * Get normal texts error.
         */
        getNormalTextsError: function () {
            var errorFields = signalRModel.errorFields(),
                selectedFields = signalRModel.selectedFields(),
                htmlMessage = $t('Error encountered while generating the following text: %s')
                    .replace('%s', '<ul>' + errorFields.map(field => `<li>${field}</li>`).join('') + '</ul>');

            if (selectedFields.length > 0 && selectedFields.length === errorFields.length) {
                self.messageSuccess(false);
                signalRModel.message(signalRModel.errorMessage());
            } else if (errorFields.length > 0) {
                signalRModel.message(self.successMessage);
                self.errorMessage(htmlMessage);
            }
        },

        /**
         * Hide text generation success message.
         */
        hideMessage: function () {
            signalRModel.showMessage(false);
        },

        /**
         * Connect to signalR stream.
         */
        connectTostream: function () {
            if (self.accessToken()) {
                self.connection = new signalR.HubConnectionBuilder()
                    .withUrl(self.signalRUrl(), {
                        accessTokenFactory: () => self.accessToken()
                    })
                    .build();
                self.connection
                    .start()
                    .then(() => self.onConnected(self.connection))
                    .catch((error) => self.onConnectionError(error));

                self.bindConnectionMessageGenerate(self.connection);

                $(window).on('focus', function () {
                    if (
                        self.connection != null &&
                        self.connection.connectionState != "Connected" &&
                        self.connection.connectionState != "Connecting"
                    ) {
                        self.connection.start();
                    }
                });
            }
        },

        /**
         * Connection is established actions.
         *
         * @param {object} connection
         * @returns {void}
         */
        onConnected: function (connection) {
            connection.send('broadcastMessage', '_SYSTEM_', 'Connected');
        },

        /**
         * Connection error actions.
         *
         * @param {object} error
         * @returns {void}
         */
        onConnectionError: function (error) {
            var doConnect = true;

            if (error.statusCode === 401 && !self.isGettingNewToken) {
                signalrUtils.getNewAccessToken(self);
            }

            if (self.enableMaxRetryAttempt && self.retryReconnectAttempt >= self.maxRetryAttempt) {
                doConnect = false;
            }

            if (doConnect) {
                if (self.retryReconnectAttempt === 0) {
                    self.connectTostream();
                } else {
                    setTimeout(function () {
                        self.connectTostream(); /* attempt to connect every 5 seconds */
                    }, 5000);
                }

                self.retryReconnectAttempt++;
            }
        },

        /**
         * Bind connection messages.
         *
         * @param {Object} connection
         * @returns {void}
         */
        bindConnectionMessageGenerate: function (connection) {
            if (connection === null) {
                /* bypass */
            }

            connection.on('broadcastMessage', self.messageCallback);
            connection.onclose(self.onConnectionError);
        },

        /**
         * Parse message.
         *
         * @param {String} encodedName
         * @param {String} encodedMsg
         *
         * @returns {Object}
         */
        createMessageEntry: function (encodedName, encodedMsg) {
            return {
                encodedName: encodedName,
                encodedMsg: JSON.parse(encodedMsg)
            };
        },

        /**
         * Process stream message.
         *
         * @param {String} name
         * @param {String} message
         *
         * @returns {void}
         */
        messageCallback: function (name, message) {
            var encodedName = name,
                encodedMsg = message
                    .replace(/&/g, '&amp;')
                    .replace(/</g, '&lt;')
                    .replace(/>/g, '&gt;'),
                messageEntry = self.createMessageEntry(encodedName, encodedMsg),
                streamRecordType = messageEntry.encodedMsg.recordType,
                streamType = messageEntry.encodedMsg.type,
                streamSubType = messageEntry.encodedMsg.subType,
                streamFieldType = messageEntry.encodedMsg.field,
                streamStop = messageEntry.encodedMsg.stop,
                streamPartialText = messageEntry.encodedMsg.partialText,
                streamRecordId = messageEntry.encodedMsg.recordId,
                streamStoreId = messageEntry.encodedMsg.storeId,
                streamIndex = messageEntry.encodedMsg.index,
                generationType = messageEntry.encodedMsg.generationType,
                indexField = '',
                streamStatus = messageEntry.encodedMsg.status,
                fieldsGenerated = {},
                productId = self.productId(),
                storeId = self.storeId(),
                entityType = self.entityType,
                streamRecordType = messageEntry.encodedMsg.recordType;

            console.log(messageEntry.encodedMsg);

            switch (streamType) {
                case self.fieldStatus:
                    if (signalRModel.generating() === true) {
                        indexField = signalrUtils.getIndexField(streamFieldType, self);
        
                        if (streamStatus === self.statusFailed) {
                            signalRModel.errorFields.push(streamFieldType);
                            signalRModel.errorMessage(messageEntry.encodedMsg.error);
                        }
        
                        if (streamFieldType !== "alt text") {
                            fieldsGenerated = signalRModel.fieldsGenerated();
                            fieldsGenerated[indexField] = true;
                            signalRModel.fieldsGenerated(fieldsGenerated);
                            if (productId === streamRecordId && storeId === streamStoreId) {
                                var statuses = textfields.statuses();
                                switch (indexField) {
                                    case 'page_title':
                                        $('.wtai-page-title-loading-mask').removeClass('wtai-show');
                                        statuses['pageTitleGenerateStatus'] = true;
                                        break;
                                    case 'page_description':
                                        $('.wtai-page-description-loading-mask').removeClass('wtai-show');
                                        statuses['pageDescriptionGenerateStatus'] = true;
                                        break;
                                    case 'product_description':
                                        $('.wtai-product-description-loading-mask').removeClass('wtai-show');
                                        statuses['productDescriptionGenerateStatus'] = true;
                                        break;
                                    case 'excerpt':
                                        $('.wtai-short-description-loading-mask').removeClass('wtai-show');
                                        statuses['productShortDescriptionGenerateStatus'] = true;
                                        break;
                                    case 'open_graph':
                                        $('.wtai-open-graph-loading-mask').removeClass('wtai-show');
                                        statuses['openGraphGenerateStatus'] = true;
                                        break;
                                }
                                
                                textfields.statuses(statuses);
                            }
                        }
        
                        if (streamFieldType === "alt text" && streamStatus === self.statusCompleted) {
                            signalRModel.imagesGenerated([...signalRModel.imagesGenerated(), ...streamRecordId.split(",")]);
                            if (productId) {
                                signalrUtils.getImages(productId, storeId, self);
                            }
                        }
                    }
                    break;
                case self.singleGenerate:
                    /** remove identifier from currently generating record identifiers if status is failed */
                    var identifier = entityType + '_' +  streamRecordId + '_' + streamStoreId;
                    var currentlyGeneratingRecordIdentifiers = currentlyGeneratingModel.recordIdentifiers();
                    if (currentlyGeneratingRecordIdentifiers.includes(identifier) && streamStatus === self.statusFailed) {
                        currentlyGeneratingRecordIdentifiers.splice(currentlyGeneratingRecordIdentifiers.indexOf(identifier), 1);
                        currentlyGeneratingModel.recordIdentifiers(currentlyGeneratingRecordIdentifiers);
                    }

                    var existingIndex = signalRModel.singleGeneratingStatuses().findIndex(function (status) {
                        return status.recordId === messageEntry.encodedMsg.recordId &&
                               status.storeId === messageEntry.encodedMsg.storeId &&
                               status.generationType === messageEntry.encodedMsg.generationType &&
                               status.field === messageEntry.encodedMsg.field;
                    });
                    if (existingIndex > -1) {
                        signalRModel.singleGeneratingStatuses.splice(existingIndex, 1, messageEntry.encodedMsg);
                    } else {
                        signalRModel.singleGeneratingStatuses.push(messageEntry.encodedMsg);
                    }

                    if (signalRModel.generating() === true) {
                        indexField = signalrUtils.getIndexField(streamFieldType, self);
                        if (productId === streamRecordId && storeId === streamStoreId) {

                            /** add identifier to currently generating record identifiers if status is not failed */
                            var currentRecordIdentifiers = currentlyGeneratingModel.recordIdentifiers();
                            var identifier = self.entityType + '_' + streamRecordId + '_' + streamStoreId;
                            if (!currentRecordIdentifiers.includes(identifier) && streamStatus !== self.statusFailed) {
                                currentRecordIdentifiers.push(identifier);
                                currentlyGeneratingModel.recordIdentifiers(currentRecordIdentifiers);
                            }

                            /* eslint-disable max-depth */
                            if (streamIndex === signalRModel.index()[indexField]) {
                                self.processSingleGenerate(
                                    streamFieldType,
                                    streamStop,
                                    streamPartialText,
                                    streamIndex
                                );
                                streamIndex += 1;
                                signalRModel.index()[indexField] = streamIndex;
        
                                while (
                                    typeof signalRModel.messages()[indexField][
                                        streamIndex
                                    ] !== 'undefined'
                                ) {
                                    self.processSingleGenerate(
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.field,
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.stop,
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.partialText,
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.index
                                    );
                                    streamIndex += 1;
                                    signalRModel.index()[indexField] = streamIndex;
                                }
                            } else {
                                signalRModel.messages()[indexField][streamIndex] =
                                    messageEntry;
                            }
        
                            if (streamStop === true) {
                                /* generate end */
                                signalRModel.index()[indexField] = 0;
                                signalRModel.messages()[indexField] = {};
                            }
                            /* eslint-enable max-depth */
                        }
                    }
                    break;
                case self.bulkGenerate:
                    if (streamRecordType === 'Category') {
                        break;
                    }
                    switch (streamSubType) {
                        case 0:
                        case 1:
                            /** remove identifier from currently generating record identifiers if status is failed */
                            var identifier = entityType + '_' +  streamRecordId + '_' + streamStoreId;
                            var currentlyGeneratingRecordIdentifiers = currentlyGeneratingModel.recordIdentifiers();
                            if (currentlyGeneratingRecordIdentifiers.includes(identifier) && streamStatus === self.statusFailed) {
                                currentlyGeneratingRecordIdentifiers.splice(currentlyGeneratingRecordIdentifiers.indexOf(identifier), 1);
                                currentlyGeneratingModel.recordIdentifiers(currentlyGeneratingRecordIdentifiers);
                            }

                            /** Suppress bulk generate progress bar if keyword analysis is ongoing */
                            if (messageEntry.encodedMsg.id.includes('KeywordOptimization') || (
                                messageEntry.encodedMsg.total === 1
                            )) {
                                var latestMessageEntry = self.latestMessageEntry();
                                latestMessageEntry[messageEntry.encodedMsg.id] = messageEntry;
                                self.latestMessageEntry(latestMessageEntry);

                                if (self.singleRequestChecker[messageEntry.encodedMsg.id] !== undefined ||
                                    self.gettingData[messageEntry.encodedMsg.id]
                                ) {
                                    return signalrUtils.processSingleRequest(
                                        messageEntry,
                                        self,
                                        self.singleRequestChecker[messageEntry.encodedMsg.id]
                                    );
                                }
                    
                                self.gettingData[messageEntry.encodedMsg.id] = $.ajax({
                                    url: self.isSingleRequestUrl,
                                    type: 'POST',
                                    data: {
                                        request_id: messageEntry.encodedMsg.id
                                    },
                                    dataType: 'json',
                                    showLoader: false,
                                    success: function (response) {
                                        self.singleRequestChecker[messageEntry.encodedMsg.id] = response.is_single_request;
                                        return signalrUtils.processSingleRequest(
                                            self.latestMessageEntry()[messageEntry.encodedMsg.id],
                                            self,
                                            response.is_single_request
                                        );
                                    },
                                    error: function (xhr, status, error) {
                                        if (status !== 'abort') {
                                            console.log(error);
                                        }
                                    }
                                });
                            } else {
                                notifications.addStreamStatus(messageEntry);
                            }
                            if (messageEntry.encodedMsg.status === 'Cancelled') {
                                signalrUtils.getCredits(self);
                            }
                            break;
                        case 2:
                            if (!messageEntry.encodedMsg.id.includes('KeywordOptimization')) {
                                if (messageEntry.encodedMsg.total === 1
                                    && messageEntry.encodedMsg.status === 'Completed'
                                    && messageEntry.encodedMsg.completed === 0) {
                                    /** do nothing */
                                    break;
                                }
                                if (messageEntry.encodedMsg.status === 'Completed' &&
                                    messageEntry.encodedMsg.completed === 1 &&
                                    messageEntry.encodedMsg.failed === 0
                                ) {
                                    self.processFailedOptimizationMessage(messageEntry);
                                } else {
                                    notifications.addStreamStatus(messageEntry);
                                }
                                keywordAnalysisProgressModel.status(messageEntry.encodedMsg);
                            }
                            if (messageEntry.encodedMsg.status === 'Completed'
                                && messageEntry.encodedMsg.completed >= 1
                                && streamRecordType === entityType) {
                                /** reload grid */
                                self.gridReload();
                            }
                            break;
                        default:
                            break;
                    }
                    break;
                case self.optimization:
                    var statuses = keywordAnalysis.statuses(),
                        storeId = messageEntry.encodedMsg.storeId,
                        recordId = messageEntry.encodedMsg.recordId,
                        existingIndex = -1;

                    statuses.forEach(function (status, index) {
                        if (Number(status.storeId) === Number(storeId) && status.recordId === recordId) {
                            existingIndex = index;
                        }
                    });

                    if (existingIndex >= 0) {
                        statuses[existingIndex] = messageEntry.encodedMsg;
                    } else {
                        statuses.push(messageEntry.encodedMsg);
                    }

                    keywordAnalysis.statuses(statuses);

                    if (messageEntry.encodedMsg.status === 'Failed'
                        && self.productId() === streamRecordId && self.storeId() === streamStoreId
                    ) {
                        if (signalRModel.generating() === true) {
                            self.repopulateTextFields();
                        }
                        /** reset optimizing id if optimization failed
                         * This will allow the generation progress bar to continue even if optimization failed
                        */
                        keywordAnalysis.optimizingId("");
                    }

                    if (messageEntry.encodedMsg.status === 'Completed'
                        && messageEntry.encodedMsg.completed >= 1
                        && streamRecordType === entityType) {
                        /** reload grid */
                        self.gridReload();
                    }
                    break;
                case self.snapshot:
                    if (productId === streamRecordId && storeId === streamStoreId) {
                        keywordAnalysis.refreshingData(true);
                        if (streamStatus === self.statusCompleted) {
                            keywordAnalysis.refreshingData(false);
                            self.getOptimization();
                        }
                    }
                    break;
                case self.openAnalysis:
                    /** $('.wtai-keyword-analysis').modal('openModal'); */
                    break;
            }
        },

        /**
         * Process single generate.
         *
         * @param {String} streamFieldType
         * @param {Boolean} streamStop
         * @param {String} streamPartialText
         * @param {Number} streamIndex
         *
         * @returns {void}
         */
        processSingleGenerate: function (
            streamFieldType,
            streamStop,
            streamPartialText,
            streamIndex
        ) {
            var currentValue = signalrUtils.getField(streamFieldType, self),
                newValue = currentValue + streamPartialText;

            /* generate starts */
            if (streamIndex === 0) {
                signalrUtils.updateField(streamFieldType, streamPartialText, self);
            }

            /* generate partial */
            if (
                streamPartialText !== null &&
                streamIndex !== null &&
                streamIndex !== 0 &&
                streamStop === false
            ) {
                signalrUtils.updateField(streamFieldType, newValue, self);
            }
        },

        /**
         * Get generated text fields.
         *
         * @returns {void}
         */
        getGenerated: function () {
            var url = self.getGeneratedUrl,
                statuses = textfields.statuses(),
                productId = self.productId(),
                storeId = self.storeId(),
                selectedFields = signalRModel.selectedFields() || [],
                imagesGenerated = signalRModel.imagesGenerated(),
                entityType = self.entityType;

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showPageTitleLoader: selectedFields.includes('page_title'),
                showPageDescriptionLoader: selectedFields.includes('page_description'),
                showProductDescriptionLoader: selectedFields.includes('product_description'),
                showShortDescriptionLoader: selectedFields.includes('short_product_description'),
                showOpenGraphLoader: selectedFields.includes('open_graph'),
                showGalleryLoader: imagesGenerated.length > 0,
                success: function (response) {
                    if (response.success) {
                        self.updateTextFields(response, selectedFields);

                        if (editData.opened()) {
                            signalRModel.showMessage(true);
                            self.showMessage(true);
                        }
                        signalRModel.generating(false);

                        /** remove identifier from currently generating record identifiers if done generating */
                        var currentRecordIdentifiers = currentlyGeneratingModel.recordIdentifiers();
                        var identifier = self.entityType + '_' +  productId + '_' + storeId;
                        if (currentRecordIdentifiers.includes(identifier)) {
                            currentRecordIdentifiers.splice(currentRecordIdentifiers.indexOf(identifier), 1);
                            currentlyGeneratingModel.recordIdentifiers(currentRecordIdentifiers);
                        }

                        if (typeof response.feedback !== 'undefined') {
                            feedbackData.feedbacks(response.feedback);
                        }

                        if (typeof response.reviewed !== 'undefined') {
                            markReviewed.reviewed(response.reviewed);
                        }
                    } else {
                        console.log(response.message);
                    }
                    
                    if (keywordAnalysis.autoGenerateOrRewriteChecked() &&
                        keywordAnalysis.optimizingId() === self.productId()
                    ) {
                        if ($('.wtai-edit-modal.wtai-edit-main').hasClass('_show')) {
                            self.getOptimization(true);
                        }
                        keywordAnalysis.autoGenerateOrRewriteChecked(false);
                    }
                }
            });
        },
        
        /**
         * Repopulate text fields.
         *
         * @returns {void}
         */
        repopulateTextFields: function () {
            var url = self.getGeneratedUrl,
                productId = self.productId(),
                storeId = self.storeId(),
                selectedFields = signalRModel.selectedFields() || [],
                imagesGenerated = signalRModel.imagesGenerated();

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showPageTitleLoader: selectedFields.includes('page_title'),
                showPageDescriptionLoader: selectedFields.includes('page_description'),
                showProductDescriptionLoader: selectedFields.includes('product_description'),
                showShortDescriptionLoader: selectedFields.includes('short_product_description'),
                showOpenGraphLoader: selectedFields.includes('open_graph'),
                showGalleryLoader: imagesGenerated.length > 0,
                success: function (response) {
                    if (response.success) {
                        self.updateTextFields(response,selectedFields);
                        signalRModel.generating(false);
                        
                        /** remove identifier from currently generating record identifiers if failed */
                        var currentRecordIdentifiers = currentlyGeneratingModel.recordIdentifiers();
                        var identifier = self.entityType + '_' + productId + '_' + storeId;
                        if (currentRecordIdentifiers.includes(identifier)) {
                            currentRecordIdentifiers.splice(currentRecordIdentifiers.indexOf(identifier), 1);
                            currentlyGeneratingModel.recordIdentifiers(currentRecordIdentifiers);
                        }

                        if (typeof response.feedback !== 'undefined') {
                            feedbackData.feedbacks(response.feedback);
                        }
                    } else {
                        console.log(response.message);
                    }
                }
            });
        },

        /**
         * Update text fields.
         *
         * @param {Array} selectedFields
         *
         * @returns {void}
         */
        updateTextFields: function (response, selectedFields) {
            var statuses = textfields.statuses();
            selectedFields.forEach((field) => {
                switch (field) {
                    case 'page_title':
                        textfields.pageTitle(
                            response.textfields.page_title
                        );
                        textfields.originalPageTitle(
                            response.textfields.page_title
                        );
                        statuses['pageTitleGenerateStatus'] = true;
                        statuses['pageTitleTransferStatus'] = false;
                        $('#wtaiPageTitleMask').addClass('wtai-generated');
                        break;
                    case 'page_description':
                        textfields.pageDescription(
                            response.textfields.page_description
                        );
                        textfields.originalPageDescription(
                            response.textfields.page_description
                        );
                        statuses['pageDescriptionGenerateStatus'] = true;
                        statuses['pageDescriptionTransferStatus'] = false;
                        $('#wtaiPageDescMask').addClass('wtai-generated');
                        break;
                    case 'product_description':
                        textfields.productDescription(
                            response.textfields.product_description
                        );
                        textfields.originalProductDescription(
                            response.textfields.product_description
                        );
                        statuses['productDescriptionGenerateStatus'] = true;
                        statuses['productDescriptionTransferStatus'] = false;
                        $('#wtaiProductDescMask + .mce-tinymce').addClass('wtai-generated');
                        $('#wtaiProductDescMask + .tox-tinymce').addClass('wtai-generated');
                        break;
                    case 'short_product_description':
                        textfields.productShortDescription(
                            response.textfields
                                .short_product_description
                        );
                        textfields.originalProductShortDescription(
                            response.textfields
                                .short_product_description
                        );
                        statuses['productShortDescriptionGenerateStatus'] = true;
                        statuses['productShortDescriptionTransferStatus'] = false;
                        $('#wtaiExcerptMask + .mce-tinymce').addClass('wtai-generated');
                        $('#wtaiExcerptMask + .tox-tinymce').addClass('wtai-generated');
                        break;
                    case 'open_graph':
                        textfields.openGraph(
                            response.textfields.open_graph
                        );
                        textfields.originalOpenGraph(
                            response.textfields.open_graph
                        );
                        statuses['openGraphGenerateStatus'] = true;
                        statuses['openGraphTransferStatus'] = false;
                        $('#wtaiOpenGraphMask').addClass('wtai-generated');
                        break;
                }
                textfields.statuses(statuses);
            });
        },

        /**
         * Get optimization.
         *
         * @returns {void}
         */
        getOptimization: function (openModal = false) {
            $.ajax({
                url: self.getOptimizationUrl,
                type: 'POST',
                data: {
                    store_id: self.storeId(),
                    record_id: self.productId(),
                    entity_type: self.entityType
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    keywordAnalysis.optimizing(false);
                    keywordAnalysis.optimized(true);
                    keywordAnalysis.optimizationData(response.api_response);
                    keywordAnalysis.snapshotData(response.api_response.snapshots || {});

                    if (openModal && keywordAnalysis.optimizationData() && keywordAnalysis.optimizationData().status === 'Completed') {
                        $('.wtai-keyword-analysis').modal('openModal');
                    }
                }
            }).always(function () {
                keywords.showProgress(false);
            });
        },

        /**
         * Get optimization list.
         *
         * @param {Object} messageEntry
         *
         * @returns {void}
         */
        processFailedOptimizationMessage: function (messageEntry) {
            var message = messageEntry.encodedMsg;
            var storeId = message.storeId;
            var status = ['Failed'];
            var recordIds = message.completedIds;
            var entityType = self.entityType;

            $.ajax({
                url: self.getOptimizationListUrl,
                type: 'POST',
                data: {
                    store_id: storeId,
                    status: status,
                    record_ids: recordIds,
                    entity_type: entityType
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (response.success) {
                        var optimizationList = response.api_response;
                        var targetRecordIdsStr = recordIds.map(String);
                        var currentUser = listingModel.currentUser();
                        
                        if (optimizationList.result.length === 0) {
                            notifications.addStreamStatus(messageEntry);
                            return;
                        }

                        var exists = optimizationList.result.some(record => targetRecordIdsStr.includes(record.recordId));
                        if (exists) {
                            notifications.addKeywordAnalysisFailedStatus(currentUser, message);
                        } else {
                            notifications.addStreamStatus(messageEntry);
                        }
                    }
                }
            });
        },

        /**
         * Reload grid.
         *
         * @returns {void}
         */
        gridReload: function () {
            var showLoader = false;

            reloadGrid.reloadUIComponent(
                'wtai_products_grid_listing.wtai_products_grid_listing_data_source',
                showLoader
            );
        }
    });
});
