/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/utils/grid/generate-image-upload',
    'WriteTextAI_WriteTextAI/js/model/grid/invalid-image-popup',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt'
], function (
    $,
    _,
    notifications,
    imageUploadUtils,
    gridInvalidImagePopupModel,
    errorMessagesModel,
    prompt
) {
    'use strict';
    
    /**
     * Bulk generate
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     *
     * @returns {void}
     */
    function bulkGenerate(self, action, data, settings)
    {
        var textTypes = [
            'page_title',
            'page_description',
            'product_description',
            'short_product_description',
            'open_graph'
        ];

        var needGalleryUpload = settings.fields.includes('alt_text'),
            needKeywordsOptimization = settings.auto_optimize_keywords,
            needThumbnailUpload = settings.attributes &&
                settings.attributes.includes('thumbnail') &&
                textTypes.some(
                    textType => settings.fields.includes(textType)
                );

        if (needThumbnailUpload || needGalleryUpload || needKeywordsOptimization) {
            imageUploadUtils.uploadImages(self, action, data, settings, needKeywordsOptimization);
        } else {
            bulkGenerateRequest(self, action, data, settings);
        }
    }

    /**
     * Bulk generate request
     * via POST request.
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     *
     * @returns {void}
     */
    function bulkGenerateRequest(self, action, data, settings)
    {
        var itemsType = '',
            selections = {};

        notifications.addNewGenerateStatus(
            data.selected,
            gridInvalidImagePopupModel.currentUser(),
            data
        );
        notifications.opened(true);

        _.extend(data.params, settings || {});

        itemsType = data.excludeMode ? 'excluded' : 'selected';

        selections[itemsType] = data[itemsType];

        if (!selections[itemsType].length) {
            selections[itemsType] = false;
        }

        _.extend(selections, data.params || {});

        /* set default store filter */
        if (!selections['filters']['store_id']) {
            selections['filters']['store_id'] = self.defaultStoreId;
        }

        $.ajax({
            url: action.url,
            type: 'POST',
            data: selections,
            dataType: 'json',
            showLoader: false,
            success: function (response) {
                if (!response.success) {
                    errorMessagesModel.messages.push(response.message);
                    notifications.statuses.remove(function (status) {
                        return status.id === notifications.tempId;
                    });
                    if (response.unauthorized) {
                        window.location.href = response.login_url;
                    }
                    if (response.reload) {
                        self.gridReload();
                    }
                } else {
                    self.gridReload();
                    prompt.submitAction('generate');
                }
            }
        });
    }

    return {
        bulkGenerate: bulkGenerate,
        bulkGenerateRequest: bulkGenerateRequest
    };
});
