/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'WriteTextAI_WriteTextAI/js/utils/grid/keyword-image-upload',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/model/grid/listing',
    'WriteTextAI_WriteTextAI/js/model/grid/popups/keyword-analysis-progress'
], function (
    $,
    keywordImageUploadUtils,
    errorMessagesModel,
    notifications,
    listingModel,
    keywordAnalysisProgressModel
) {
    'use strict';

    /**
     * Bulk keyword analysis
     *
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     *
     * @returns {void}
     */
    function bulkKeywordAnalysis(self, action, data, settings)
    {
        keywordImageUploadUtils.uploadImages(self, action, data, settings, true);
    }

    /**
     * Bulk keyword analysis request
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     *
     * @returns {void}
     */
    function bulkKeywordAnalysisRequest(self, action, data, settings)
    {
        var itemsType = '',
            selections = {};

        keywordAnalysisProgressModel.status({});

        _.extend(data.params, settings || {});

        itemsType = data.excludeMode ? 'excluded' : 'selected';

        selections[itemsType] = data[itemsType];

        if (!selections[itemsType].length) {
            selections[itemsType] = false;
        }

        _.extend(selections, data.params || {});

        /* set default store filter */
        if (!selections['filters']['store_id']) {
            selections['filters']['store_id'] = self.defaultStoreId;
        }
                    
        notifications.addNewKeywordAnalysisStatus(
            data.selected,
            listingModel.currentUser(),
            data
        );
        notifications.opened(true);

        $.ajax({
            url: action.url,
            type: 'POST',
            data: selections,
            dataType: 'json',
            showLoader: false,
            success: function (response) {
                if (!response.success) {
                    errorMessagesModel.messages.push(response.message);
                    notifications.statuses.remove(function (status) {
                        return status.id === notifications.koTempId;
                    });
                    if (response.unauthorized) {
                        window.location.href = response.login_url;
                    }
                    
                    if (response.reload) {
                        self.gridReload();
                    }
                } else {
                    self.gridReload();
                }
            }
        });
    }

    return {
        bulkKeywordAnalysis: bulkKeywordAnalysis,
        bulkKeywordAnalysisRequest: bulkKeywordAnalysisRequest
    };
});
