/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/grid/upload-images',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/keyword-invalid-image-popup'
], function (
    $,
    _,
    uploadImagesModel,
    referenceProducts,
    errorMessagesModel,
    keywordInvalidImagePopupModel
) {
    'use strict';

    /**
     * Main function for uploading images
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     * @param {Boolean} needKeywordsOptimization - Whether keywords optimization is needed.
     *
     * @returns {void}
     */
    function uploadImages(self, action, data, settings, needKeywordsOptimization = false)
    {
        $('.wtai-upload-image-loader-ideas').modal('openModal');
        uploadImagesModel.failedThumbnails([]);
        uploadImagesModel.failedProducts([]);
        uploadImagesModel.successThumbnails([]);
        uploadThumbnails(self, action, data, settings, 0, needKeywordsOptimization);
    }

    /**
     * Upload thumbnails
     *
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     * @param {Number} index - Index of the current chunk.
     * @param {Boolean} needKeywordsOptimization - Whether keywords optimization is needed.
     *
     * @returns {void}
     */
    function uploadThumbnails(self, action, data, settings, index = 0, needKeywordsOptimization = false)
    {
        var selections = {};
        var endIndex = Math.min(index + 10, data.selected.length);

        if (needKeywordsOptimization && index < data.selected.length) {
            $.ajax({
                url: self.uploadThumbnailsUrl,
                type: 'POST',
                data: selections,
                data: {
                    selected: data.selected.slice(index, endIndex),
                    store_id: referenceProducts.storeId()
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        $('.wtai-upload-image-loader').modal('closeModal');
                        $('.wtai-upload-image-loader-ideas').modal('closeModal');
                        $('.modals-overlay.wtai-upload-image-loader-overlay').remove();
                        errorMessagesModel.messages.push(response.message);

                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                    } else {
                        uploadImagesModel.failedThumbnails.push(...response.failed_thumbnails);
                        uploadImagesModel.failedProducts.push(...response.failed_products);
                        uploadImagesModel.successThumbnails.push(...response.success_thumbnails);

                        /** Upload next chunk */
                        uploadThumbnails(self, action, data, settings, endIndex, needKeywordsOptimization);
                    }
                }
            });
        } else {
            $('.wtai-upload-image-loader').modal('closeModal');
            $('.wtai-upload-image-loader-ideas').modal('closeModal');
            $('.modals-overlay.wtai-upload-image-loader-overlay').remove();

            data.params.uploaded_images = uploadImagesModel.successThumbnails();
            if (validateInvalidImages()) {
                self.bulkKeywordAnalysisRequest(self, action, data, settings);
            } else {
                keywordInvalidImagePopupModel.self(self);
                keywordInvalidImagePopupModel.action(action);
                keywordInvalidImagePopupModel.data(data);
                keywordInvalidImagePopupModel.settings(settings);
                $('.wtai-grid-keyword-invalid-image-popup').modal('openModal');
            }
        }

        /**
         * Validate invalid images
         */
        function validateInvalidImages()
        {
            var failedThumbnails = uploadImagesModel.failedThumbnails();
    
            return failedThumbnails.length === 0;
        }
    }

    return {
        uploadImages: uploadImages,
        uploadThumbnails: uploadThumbnails
    };
});
